@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2013-2014 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -17,12 +17,9 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include "mrn_condition_converter.hpp"
 #include "mrn_time_converter.hpp"
+#include "mrn_smart_grn_obj.hpp"
 
 // for debug
 #define MRN_CLASS_NAME "mrn::ConditionConverter"
@@ -355,6 +352,11 @@ namespace mrn {
     case MYSQL_TYPE_GEOMETRY:
       type = UNSUPPORTED_TYPE;
       break;
+#ifdef MRN_HAVE_MYSQL_TYPE_JSON
+    case MYSQL_TYPE_JSON:
+      type = STRING_TYPE;
+      break;
+#endif
     }
 
     DBUG_RETURN(type);
@@ -371,10 +373,10 @@ namespace mrn {
     if (!column) {
       DBUG_RETURN(false);
     }
+    mrn::SmartGrnObj smart_column(ctx_, column);
 
     int n_indexes = grn_column_index(ctx_, column, _operator, NULL, 0, NULL);
     bool convertable = (n_indexes > 0);
-    grn_obj_unlink(ctx_, column);
 
     DBUG_RETURN(convertable);
   }
@@ -407,11 +409,11 @@ namespace mrn {
     DBUG_RETURN(have);
   }
 
-  const Item_func *ConditionConverter::find_match_against(const Item *item) {
+  unsigned int ConditionConverter::count_match_against(const Item *item) {
     MRN_DBUG_ENTER_METHOD();
 
     if (!item) {
-      DBUG_RETURN(NULL);
+      DBUG_RETURN(0);
     }
 
     switch (item->type()) {
@@ -419,14 +421,13 @@ namespace mrn {
       if (is_storage_mode_) {
         Item_cond *cond_item = (Item_cond *)item;
         if (cond_item->functype() == Item_func::COND_AND_FUNC) {
+          unsigned int n_match_againsts = 0;
           List_iterator<Item> iterator(*((cond_item)->argument_list()));
           const Item *sub_item;
           while ((sub_item = iterator++)) {
-            const Item_func *match_against = find_match_against(sub_item);
-            if (match_against) {
-              DBUG_RETURN(match_against);
-            }
+            n_match_againsts += count_match_against(sub_item);
           }
+          DBUG_RETURN(n_match_againsts);
         }
       }
       break;
@@ -435,7 +436,7 @@ namespace mrn {
         const Item_func *func_item = (const Item_func *)item;
         switch (func_item->functype()) {
         case Item_func::FT_FUNC:
-          DBUG_RETURN(func_item);
+          DBUG_RETURN(1);
           break;
         default:
           break;
@@ -446,7 +447,7 @@ namespace mrn {
       break;
     }
 
-    DBUG_RETURN(NULL);
+    DBUG_RETURN(0);
   }
 
   void ConditionConverter::convert(const Item *where, grn_obj *expression) {
@@ -528,13 +529,13 @@ namespace mrn {
     append_field_value(field_item, expression);
 
     grn_obj include;
+    mrn::SmartGrnObj smart_include(ctx_, &include);
     GRN_TEXT_INIT(&include, 0);
     GRN_TEXT_PUTS(ctx_, &include, "include");
     append_const_item(field_item, min_item, expression);
     grn_expr_append_const(ctx_, expression, &include, GRN_OP_PUSH, 1);
     append_const_item(field_item, max_item, expression);
     grn_expr_append_const(ctx_, expression, &include, GRN_OP_PUSH, 1);
-    grn_obj_unlink(ctx_, &include);
 
     grn_expr_append_op(ctx_, expression, GRN_OP_CALL, 5);
 
